"""Plotly visualization components for fundedness."""

from fundedness.viz.colors import COLORS
from fundedness.viz.comparison import create_strategy_comparison_chart
from fundedness.viz.fan_chart import create_fan_chart
from fundedness.viz.histogram import create_time_distribution_histogram
from fundedness.viz.optimal import (
    create_optimal_allocation_curve,
    create_optimal_policy_summary,
    create_optimal_spending_curve,
    create_sensitivity_heatmap,
    create_spending_comparison_by_age,
    create_utility_comparison_chart,
)
from fundedness.viz.survival import create_survival_curve
from fundedness.viz.tornado import create_tornado_chart
from fundedness.viz.waterfall import create_cefr_waterfall

__all__ = [
    "COLORS",
    "create_cefr_waterfall",
    "create_fan_chart",
    "create_optimal_allocation_curve",
    "create_optimal_policy_summary",
    "create_optimal_spending_curve",
    "create_sensitivity_heatmap",
    "create_spending_comparison_by_age",
    "create_strategy_comparison_chart",
    "create_survival_curve",
    "create_time_distribution_histogram",
    "create_tornado_chart",
    "create_utility_comparison_chart",
]
